﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Collections;
using Platform.Net;

namespace Platform.Geo
{
    public class GeoCollection : GeoShape, IList<GeoShape>
    {
        // list of shapes
        private List<GeoShape> _Shapes = 
            new List<GeoShape>();

        #region WKT conversion

        /// <summary>
        /// Convert to WKT
        /// </summary>
        /// <returns></returns>
        public override string ToWKT()
        {
            return "GEOMETRYCOLLECTION(" + string.Join(",", _Shapes.Select(s => s.ToWKT())) + ")";
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoCollection FromWKT(string wkt)
        {
            // create reader
            var reader = new NetStringReader(wkt);
            reader.Skip();

            // check for geometry collection
            if (reader.ReadPattern("GEOMETRYCOLLECTION") == -1)
                throw new FormatException("Missing literal 'GEOMETRYCOLLECTION' in wkt ");

            // skip white space
            reader.Skip();

            // read opening bracket
            if ( !reader.Read('(') )
                throw new FormatException("Missing literal '(' in wkt ");

            // create new collection
            var result = new GeoCollection();

            // read all shapes
            while (true)
            {
                // skip white space
                reader.Skip();

                // check polygon types
                switch (reader.StartsWith("POINT", "LINESTRING", "POLYGON", "MULTIPOLYGON"))
                {
                    case 0:
                        result.Add(GeoPosition.FromWKT(reader));
                        break;
                    case 1:
                        result.Add(GeoPolyline.FromWKT(reader));
                        break;
                    case 2:
                        result.Add(GeoPolygon.FromWKT(reader));
                        break;
                    case 3:
                        result.Add(GeoPolygon.FromWKT(reader));
                        break;
                    default:
                        throw new FormatException("Missing literal 'POINT', 'LINESTRING', 'POLYGON' or 'MULTIPOLYGON' in wkt ");
                }

                // read white space
                reader.Skip();

                // check for next item
                int pattern = reader.ReadPattern(",", ")");
                if( pattern == -1 ) throw new FormatException("Missing literal ',' or ')' in wkt ");
                if (pattern == 1) break;
            }

            // return result
            return result;
        }

        #endregion

        #region Tools

        /// <summary>
        /// Get bouding rectangle
        /// </summary>
        /// <returns></returns>
        public override GeoRectangle GetBounds()
        {
            GeoRectangle bounds = null;
            foreach( var item in _Shapes )
                if (bounds == null) bounds = item.GetBounds();
                else bounds = bounds.Union(item.GetBounds());
            return bounds;
        }

        #endregion

        #region IList Implementation

        public GeoShape this[int index]
        {
            get { return _Shapes[index]; }
            set { _Shapes[index] = value; }
        }

        public int Count
        {
            get { return _Shapes.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public void Add(GeoShape item)
        {
            _Shapes.Add(item);
        }

        public void Clear()
        {
            _Shapes.Clear();
        }

        public bool Contains(GeoShape item)
        {
            return _Shapes.Contains(item);
        }

        public void CopyTo(GeoShape[] array, int arrayIndex)
        {
            _Shapes.CopyTo(array, arrayIndex);
        }

        public IEnumerator<GeoShape> GetEnumerator()
        {
            return _Shapes.GetEnumerator();
        }

        public int IndexOf(GeoShape item)
        {
            return _Shapes.IndexOf(item);
        }

        public void Insert(int index, GeoShape item)
        {
            _Shapes.Insert(index, item);
        }

        public bool Remove(GeoShape item)
        {
            return _Shapes.Remove(item);
        }

        public void RemoveAt(int index)
        {
            _Shapes.RemoveAt(index);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return _Shapes.GetEnumerator();
        }

        #endregion
    }
}
